# library(devtools)
# install_github("antongrau/eliter")
library(tidyverse)
library(plyr)
library(eliter)
library(readr)
library(RColorBrewer)
library(ggrepel)
library(viridis)
library(ggplot2)
library(ggthemes)
library(forcats)


slice_to_year <- function(g, year, delete.vertices = TRUE){
  active      <- E(g)$Start <= year & E(g)$End >= year
  g           <- delete.edges(g, which(active == FALSE))
  
  active.vertices <- V(g)$Start <= year & V(g)$End >= year
  
  if (delete.vertices == TRUE) {
  
  if (any(active.vertices == FALSE)) {
    g           <- delete.vertices(g, which(active.vertices == FALSE))
  }
  }  
    
  g
}


sending.MB <- function(g){
  g.s <- delete.edges(g, which(E(g)$"Director A is MB" == FALSE))
  g.s <- igraph::simplify(g.s)
  degree(g.s, mode = "out")
}

sending.union <- function(g){
  g.s <- delete.edges(g, which(E(g)$"Type of Director Comp A" != "Employee Rep SB"))
  g.s <- igraph::simplify(g.s)
  degree(g.s, mode = "out")
}

sending.bank <- function(g){
  g.s <- delete.edges(g, which(is.na(E(g)$"Bank managing dir or chairman SB")))
  g.s <- igraph::simplify(g.s)
  degree(g.s, mode = "out")
}

com.no.bank <- function(g){
  g.s <- delete.edges(g, which(!is.na(E(g)$"Bank managing dir or chairman SB")))
  g.s <- largest.component(g.s)
  vcount(g.s)
}  
  
com.no.union <- function(g){
  g.s <- delete.edges(g, which(E(g)$"Type of Director Comp A" == "Employee Rep SB"))
  g.s <- largest.component(g.s)
  vcount(g.s)
}  



activity.plot <- function(g, ...){
  sc        <- list()
  sc$size   <- scale_size_continuous(range = c(1,8), name = "Activity in period")
  sc$fill   <- scale_fill_gradientn(colours = brewer.pal(6, "YlGn"), name = "Activity in period")
  sc$alpha  <- scale_alpha_discrete(range = c(0.5, 1), name = "Multiple tie")
  sc$color  <- scale_color_manual(values = c("#78c679", "black"), name = "Sends MB")
  
  p     <- graph.plot(g, text = TRUE, vertex.size = V(g)$degree.period, vertex.fill = V(g)$degree.period, edge.alpha = is.multiple(g),
                      edge.color = E(g)$"Director A is MB",
                      edge.size = 0.3, midpoints = TRUE, ...)
  p + sc
}

activity.plot.small <- function(g, ...){
  sc        <- list()
  sc$size   <- scale_size_manual(values = c(0.8, 1.5), guide = "none")
  sc$fill   <- scale_fill_manual(values = c("white", "black"), guide = "none")
  sc$alpha  <- scale_alpha_discrete(range = c(0.5, 1), name = "Multiple tie")
  sc$space  <- theme(plot.margin = unit(c(0, 0, 0, 0), "cm"))
  #sc$color  <- scale_color_manual(values = c("#78c679", "black"), name = "Sends MB")
  g <- igraph::simplify(g)
  g <- as.undirected(g)
  
  set <- c("BASF", "Bayer", "Hoechst")
  
  f     <- V(g)$name %in% set
  
  p     <- graph.plot(g, text = FALSE, vertex.size = f, vertex.fill = f, edge.alpha = 0.3,
                      edge.color = "black",
                      edge.size = 0.5, midpoints = FALSE
                      #, ...
                      )
  p + sc
}

layout_with_isolates <- function(g){
  
  # Hvis man fixerer layoutet for isolates så kan det være de andre punkter flytter sig fint iforhold til dem
  
  g
  
  
  
}


activity.time.series <- function(graph.all){
  
  lay                <- layout_with_kk(simplify(graph.all))
  gl                 <- lapply(period, slice_to_year, g = graph.all, delete.vertices = FALSE)  
  lapply(gl, activity.plot, layout = lay)
}



get.ego.from.slices   <- function(slices, nodes, ...){
  ego.slices          <- lapply(slices, make_ego_graph, order = 1, nodes = nodes, ...)
  ego.slices          <- lapply(ego.slices, "[[", 1)
  ego.slices
}

combine.slices        <- function(slices){
  
  add.to.names <- function(x, name){
    V(x)$phase    <- rep(name, vcount(x))
    V(x)$old.name <- V(x)$name
    V(x)$name     <- paste(V(x)$name, name)
    x
  }
  
  for (i in 1:length(slices)) slices[[i]] <- add.to.names(slices[[i]], names(slices)[i])
  
  em <- lapply(slices, function(x) data.frame(get.edgelist(x, names = TRUE), edge.attributes(x), check.names = FALSE, stringsAsFactors = FALSE))
  vm <- lapply(slices, function(x) data.frame(vertex.attributes(x), stringsAsFactors = FALSE, check.names = FALSE))
  em <- do.call(rbind, em)
  em$type <- TRUE
  
  vm <- dplyr::bind_rows(vm)
  
  graph_from_data_frame(em, directed = FALSE, vertices = vm)
  
}

connect.nodes.per.phase  <- function(combined.graph){
  
  vm       <- data.frame(vertex.attributes(combined.graph), stringsAsFactors = FALSE, check.names = FALSE)
  s.vm     <- split(vm, f = vm$old.name)
  rows     <- sapply(s.vm, nrow)
  
  s.vm     <- s.vm[which(rows > 1)]  
  edges    <- lapply(s.vm, function(x) as.vector(t(cbind(x$name[-length(x$name)], x$name[-1]))))
  #edges    <- do.call("c", edges)
  
  for (i in 1:length(edges)) combined.graph <- add_edges(combined.graph, edges = edges[[i]])
  
  #combined.graph <- add_edges(combined.graph, edges = edges)
  combined.graph
}

cuts.the.flat.circle <- function(combined.graph){
  phase        <- as.numeric(V(combined.graph)$phase)
  names(phase) <- V(combined.graph)$name  
  el           <- get.edgelist(combined.graph)
  el           <- data.frame(el, a = phase[el[,1]], b = phase[el[,2]])
  el$a - el$b  < -1
}

add.measure.to.slices <- function(slices, label = NULL, FUN = degree, ...){
  if(is.null(label)) label <- deparse(substitute(FUN))
  
  add.measure <- function(x, label = NULL, FUN, ...) set.vertex.attribute(x, name = label, value = FUN(x,  ...))
  
  purrr::map(.x = slices, .f = add.measure,  FUN = FUN, label = label)
}

heatmap           <- function(graph){
  
  graph <- as.undirected(graph)
  adj             <- as_adj(graph, attr = "weight")
  rs              <- rowSums(adj != 0)
  #adj             <- adj[order(rs, decreasing = TRUE), order(rs, decreasing = FALSE)]
  md              <- melt(as.matrix(adj))
  
  md              <- as.matrix(adj) %>% as_tibble()
  md$label        <- colnames(md) 
  md              <- md %>% gather(key = "x", value = "value", -label)
  md$label        <- as_factor(md$label) %>%  fct_relevel(names(sort(rs, decreasing = TRUE)))
  md$x            <- as_factor(md$x) %>%  fct_relevel(rev(names(sort(rs, decreasing = TRUE))))
  
  
  
  md$value_cat    <- cut(md$value, breaks = c(-Inf, 0, 5, 10, 20, Inf), include.lowest = T) %>%
    fct_recode("0" = "[-Inf,0]", "1-5" = "(0,5]", "6-10" = "(5,10]", "11-20" = "(10,20]", "+20" = "(20, Inf]")
  
  sc             <- list()
  sc$axis.text   <- theme(axis.text.x = element_text(angle = -90, hjust = 1))
  sc$coord.equal <- coord_equal()
  sc$scale       <- scale_x_discrete(position = "top")
  sc$xlab        <- xlab(label = "")
  sc$ylab        <- ylab(label = "")
  sc$fill        <- scale_fill_viridis(option = "magma", discrete = TRUE, name = "Years") 
  sc$fill        <- scale_fill_manual(values = c("black", rev(brewer.pal(4, "YlGnBu"))), name = "Years connected")
  sc$title       <- theme(plot.title  = element_text(hjust = 0)) 
  
  p              <- ggplot(md, aes(x = fct_rev(x), y = fct_rev(label), fill = value_cat)) + geom_tile(color = "grey90") + theme_tufte()
  p + sc
}


accumulative.adjacency.list <- function(spell.graph, start = min(E(spell.graph)$start), end = max(E(spell.graph)$end), steps = 12){
  
  del          <- which(E(spell.graph)$start > end | E(spell.graph)$end < start)
  g            <- delete.edges(spell.graph, del)
  
  period       <- start:end
  adj.cum      <- get.adjacency(period.graph(g, start = period[1], end = period[1]))
  pb           <- txtProgressBar(min = 2, max = length(period), style = 3)
  
  adj.list       <- list()
  adj.list[[1]]  <- adj.cum
  i.seq          <- seq(from = 1, to = length(period), by = steps)
  for (i in i.seq[-1]) {
    t            <- period[i]
    gp           <- period.graph(g, start = t, end = t)
    adj.t        <- get.adjacency(gp)
    adj.cum      <- adj.cum + adj.t
    adj.list     <- c(adj.list, adj.cum)
    
    setTxtProgressBar(pb, i)
  }
  close(pb)
  names(adj.list) <- period[i.seq]
  adj.list
}

accumulative.adjacency.list <- function(spell.graph, start = min(E(spell.graph)$start), end = max(E(spell.graph)$end), steps = 12){
  
  del          <- which(E(spell.graph)$start > end | E(spell.graph)$end < start)
  g            <- delete.edges(spell.graph, del)
  
  period       <- start:end
  adj.cum      <- get.adjacency(period.graph(g, start = period[1], end = period[1]))
  pb           <- txtProgressBar(min = 2, max = length(period), style = 3)
  
  adj.list       <- list()
  adj.list[[1]]  <- adj.cum
  i.seq          <- seq(from = 1, to = length(period), by = steps)
  for (i in i.seq[-1]) {
    t            <- period[i]
    gp           <- period.graph(g, start = t, end = t)
    adj.t        <- get.adjacency(gp)
    adj.cum      <- adj.cum + adj.t
    adj.list     <- c(adj.list, adj.cum)
    
    setTxtProgressBar(pb, i)
  }
  close(pb)
  names(adj.list) <- period[i.seq]
  adj.list
}

  past.and.present.graph <- function(year, graph.present, graph.present.reach, graph.past){
  
  present <- simplify(graph.present[[year]])
  reach   <- simplify(graph.present.reach[[year]])
  past    <- simplify(graph.past[[year]])
  
  
  edge.types  <- list()
  edge.types$present   <- data.frame(get.edgelist(present))
  edge.types$reach     <- data.frame(get.edgelist(reach - present))
  edge.types$past      <- data.frame(get.edgelist(past - reach - present))
  
  edges                <- bind_rows(edge.types, .id = "type")
  edges                <- edges[,c(2,3, 1)]
  
  g                    <- graph_from_data_frame(edges, directed = FALSE)
  if(length(difference(V(present), V(g))) > 0) g <- g + difference(present, g) # add isolates
  
  g
  }


  slice.den.to.year <- function(den, year = 1970){
    active.positions  <- den$PERSON_END >= year & den$PERSON_START <= year
    den.active        <- den[active.positions,]
    past.position     <- den$PERSON_END < year
    den.past          <- den[past.position, ]
    active.names      <- unique(den.active$NAME)
    den.active.past   <- den.past[den.past$NAME %in% active.names,]
    out               <- bind_rows(den.active, den.active.past)
    out$active        <- out$POSITION_ID %in% den.active$POSITION_ID
    out
  }
  
  
  incidence.with.former.collegaues <- function(a.list, den, year = "1950"){
    a           <- a.list[[year]]
    g           <- graph_from_adjacency_matrix(a, weighted = T, mode = "undirected")
    den.active  <- den[year(den$PERSON_END) >= year & year(den$PERSON_START) <= year,]
    active.dir  <- unique(den.active$NAME)
    g           <- g - which((V(g)$name %in% active.dir) == FALSE)
    el          <- get.edgelist(g)
    
    incidence   <- xtabs(~NAME + AFFILIATION, den.active, sparse = TRUE)
    incidence.past <- incidence
    
    incident.friends <- function(incidence, x){
      incidence[x, (incidence[x[1],] + incidence[x[2],]) > 0] <- 1
      incidence
    }
    
    for(i in 1:nrow(el)) incidence.past[el[i,],] <- (incidence.past[el[i,],] + incident.friends(incidence, el[i,])[el[i,], ]) - incidence.past[el[i,],]
    incidence.past
  }
  
  incidence.active <- function(den, year = "1980"){
    den.active  <- den[year(den$PERSON_END) >= year & year(den$PERSON_START) <= year,]
    incidence   <- xtabs(~NAME + AFFILIATION, den.active, sparse = TRUE)
  }
  
  flat.circle.plot <- function(slices, nodes, ...){
    
    slices         <- get.ego.from.slices(slices, nodes)
    
    r <- rle(purrr::map_int(slices, vcount))
    
    # Trim start
    if((r$values == 1)[1] == TRUE){
      slices[1:r$length[1]] <- NULL
    }
    
    r <- rle(purrr::map_int(slices, vcount))  
    
    # Trim end
    if( tail(r$values, 1) == 1){
      l <- length(slices)
      s <- (l - tail(r$length, 1)) : l
      slices[s] <- NULL
    }
    
    set.type.of.director <- function(x, nodes){
      ed <- as.data.frame(get.edge.attribute(x))
      ed <- cbind(data.frame(get.edgelist(x), stringsAsFactors = FALSE), ed)
      #ed <- ed[ed$X2 == nodes,]
      
      ed$Type.of.Director.Comp.A[is.na(ed$Type.of.Director.Comp.A)] <- "Other"
      ed$Type.of.Director.Comp.B[is.na(ed$Type.of.Director.Comp.B)] <- "Other"
      
      ed$is.chair <- ed$Type.of.Director.Comp.A == "Chairman" | ed$Type.of.Director.Comp.B == "Chairman"
      ed$is.ceo   <- ed$Type.of.Director.Comp.A %in% c("CEO", "Executive") | ed$Type.of.Director.Comp.B %in% c("CEO", "Executive")
      
      ed$top.pos  <- (ed$is.ceo * 10) + ed$is.chair
      
      gd <- ed %>% group_by(X1) %>% dplyr::summarise(role = max(top.pos))
      gd$role <- gd$role %>% as.character() %>% fct_recode("CEO/Executive" = "10", "CEO/Executive" = "11", "Chairman" = "1", "Other" = "0")
      
      x  <- set.vertex.attribute(x, "type.of.dir", index = gd$X1, value = as.character(gd$role))
      x  
    }
    
    edge.to.ego <- function(x, nodes){
      
      set <- incident(x, nodes)
      if (any(is.na(as.numeric(set))) == FALSE){
        x <- set.edge.attribute(x, "is.ego.edge", set, TRUE)
      }
      x
    }
    
    
    slices         <- purrr::map(slices, edge.to.ego, nodes = nodes)
    
    slices         <- purrr::map(slices, set.type.of.director, nodes = nodes)
    
    combined.graph <- combine.slices(slices)
    combined.graph <- connect.nodes.per.phase(combined.graph)
    
    
    
    g               <- combined.graph
    #g     <- delete.edges(g, which(cuts.the.flat.circle(g)))
    E(g)$weight     <- (cuts.the.flat.circle(g) * 1) + 1
    E(g)$weight     <- E(g)$weight + (is.na(E(g)$type) * 1.5) 
    
    vm       <- data.frame(vertex.attributes(combined.graph), stringsAsFactors = FALSE, check.names = FALSE)
    
    
    ego  <- which(V(combined.graph)$old.name == nodes)
    #vm       <- vm[-del.ego, ]
    
    vf       <- as.numeric(V(combined.graph)$phase)
    vm       <- group_by(vm, old.name) %>% dplyr::mutate(count = n())
    vf       <- vm$count
    vm$one   <- 1
    vm       <- group_by(vm, old.name) %>% dplyr::mutate(n_years = cumsum(one))
    vf       <- vm$n_years
    
    vt        <- V(combined.graph)$old.name
    vt        <- as.character(as.numeric(factor(vt, levels = unique(vm$old.name), ordered = T)))
    
    end.phase     <- group_by(vm, old.name) %>% select(old.name, phase) %>% top_n(1, phase)
    colnames(end.phase) <- c("old.name", "end.phase")
    vm            <- left_join(vm, end.phase, by = "old.name")
    
    phase <- as.numeric(V(g)$phase)
    phase <- phase - min(phase) + 1
    l     <- max(phase)
    
    ls <- cbind(round(phase, -1), ((phase-l) *  -1))
    #ls <- ls * 10
    
    fr.weight <- 1/E(g)$weight
    fr.weight[E(g)$weight == 1] <- 0.5
    
    fr.weight[E(g)$is.ego.edge == FALSE] <- 0
    
  
    lay       <- layout_with_fr(g, coords = ls * 1, weights = fr.weight, start.temp = sqrt(vcount(g) * 1) * 1)
    lay       <- norm_coords(lay[, 1:2], xmin = 1, xmax = 10^10, ymin = 1, ymax = 10^10)
    
    
    first.old <- duplicated(vm$old.name) == FALSE
    lay.old   <- data.frame(lay)
    lay.old$name.old <- paste0(vt, ". ", vm$name, ":", vm$end.phase)
    lay.old$name.old[first.old == FALSE] <- NA
    
    
    ec    <- as.numeric(is.na(E(combined.graph)$type) == TRUE)
    ec[which(E(combined.graph)$is.ego.edge)] <- -1
    
    sc        <- list()
    sc$color  <- scale_color_manual(values = c("-1" = "black", "0" = "#b30000", "1" = "grey30"), name = "Time tie", guide = "none")
    #sc$fill   <- scale_fill_gradientn(colours = brewer.pal(9, name = "YlOrRd"), name = "Degree in current year")
    sc$fill   <- scale_fill_manual(values = c("CEO/Executive" = "#41b6c4", "Chairman" = "#fec44f", "Director" = "white"), name = "Type of director")
    sc$shape  <- scale_shape_manual(values = c(21, 22, 25, 24), name = "Type of director" )
    sc$size   <- scale_size_continuous(range = c(1.2, 3.5), name = "Degree in current year")
    sc$line   <- scale_linetype(guide = "none")
    sc$alpha  <- scale_alpha_manual(guide = "none", values = c(0.5, 1, 0.5))
    
    
    fill.fact <- factor(V(combined.graph)$type.of.dir) %>% fct_explicit_na(na_level = "Other") %>% fct_recode("Director" = "Other")
    
    p <- graph.plot(combined.graph, layout = lay, vertex.fill = fill.fact, vertex.shape = 21,
                    vertex.color = "white", vertex.size = 3, 
                    edge.color = factor(ec), edge.line =  as.factor(E(g)$weight), edge.alpha = factor(ec), edge.size = 0.3, vertex.alpha = 0.5,
                    text = vt, text.vjust = 0.5, text.size = 2.5, text.family = "sans")
    
    
    lay.old$name.old[is.na(lay.old$name.old)] <- ""
    p <- p + geom_label_repel(data = lay.old, mapping = aes(x = X1, y = X2, label = name.old), size = 2.5, box.padding = unit(0, units = "cm"), label.size = 0, segment.color = "grey70", segment.size = 0.5, point.padding = unit(1, "lines"))
    p <- p + sc + theme(
      legend.position = "bottom",
      legend.justification = c("right", "top"),
      legend.box.just = "left",
      legend.margin = margin(6, 6, 6, 6)
    )
    p
  }
  
  connected.by.set <- function(g, set){
    v            <- 1:vcount(g)
    set.node     <- min(v[V(g)$name %in% set])
    v[V(g)$name %in% set] <- set.node
    
    out <- c(NA, NA, NA)
    if(set.node != Inf){
      g.c          <- contract.vertices(g, v)
      set.node.id  <- V(g.c)[set.node]
      g.e          <- make_ego_graph(g.c, order = 1, nodes = set.node.id)[[1]]
      is.set       <- sapply(V(g.e)$name, function(x, set) any(x %in% set), set = set)
      set.connected.to <- sum(g.e[is.set,] > 0)
      g.d          <- delete.vertices(g.e, which(is.set))
      neighbors.already.connected <- sum(degree(g.d) > 0)
      out <- c("Connected to" = set.connected.to, "Already connected" = neighbors.already.connected, "Unconnected" = set.connected.to - neighbors.already.connected)
    }
    t(out)
  }
  